<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @package    Easy_Translate
 * @subpackage Easy_Translate/admin
 */
class Easy_Translate_Admin
{

    /**
     * The capability required for menu to be displayed to the user.
     */
    const CAPABILITY = 'manage_options';

    /**
     * The slug name to refer to menu.
     */
    const MENU_MAIN = 'admin-easy-translate';
    const MENU_PROJECTS = 'admin-easy-translate-projects';
    const MENU_INTEGRATION = 'admin-easy-translate-integration';
    const MENU_PARTIAL_TRANSLATIONS = 'admin-easy-translate-partial-translations';

    /**
     * API integration object.
     */
    public $api_integration;

    /**
     * Project object.
     */
    public $projects;

    /**
     * @var Easy_Translate_String_Translation_List.
     */
    private $string_translation_list;

    /**
     * @var Easy_Translate_Post_Translation_List.
     */
    private $post_translation_list;

    /**
     * Easy_Translate_Admin constructor.
     */
    public function __construct()
    {
        $this->api_integration = new Easy_Translate_API_Integration();
        $this->projects = new Easy_Translate_Projects();
    }

    /**
     * Register the stylesheets for the admin area.
     */
    public function enqueue_styles()
    {
        /**
         * This function is provided for demonstration purposes only.
         * An instance of this class should be passed to the run() function
         * defined in Easy_Translate_Loader as all of the hooks are defined
         * in that particular class.
         * The Easy_Translate_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        wp_enqueue_style(EASY_TRANSLATE, plugin_dir_url(__FILE__) . 'css/easy-translate-admin.css', [],
            EASY_TRANSLATE_VERSION, 'all');
    }

    /**
     * Register the JavaScript for the admin area.
     */
    public function enqueue_scripts()
    {
        /**
         * This function is provided for demonstration purposes only.
         * An instance of this class should be passed to the run() function
         * defined in Easy_Translate_Loader as all of the hooks are defined
         * in that particular class.
         * The Easy_Translate_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        wp_enqueue_script(EASY_TRANSLATE, plugin_dir_url(__FILE__) . 'js/easy-translate-admin.js', ['jquery'],
            EASY_TRANSLATE_VERSION, false);
    }

    /**
     * Adds menu pages.
     */
    public function admin_menu_setup()
    {
        add_menu_page(
            __('EasyTranslate', EASY_TRANSLATE),
            __('EasyTranslate', EASY_TRANSLATE),
            self::CAPABILITY,
            self::MENU_MAIN,
            [$this, 'show_translations_page'],
            plugin_dir_url(__FILE__) . 'img/icon.png'
        );

        add_submenu_page(
            self::MENU_MAIN,
            __('Page translations', EASY_TRANSLATE),
            __('Page translations', EASY_TRANSLATE),
            self::CAPABILITY,
            self::MENU_MAIN,
            [$this, 'show_translations_page']
        );

        add_submenu_page(
            self::MENU_MAIN,
            __('String translations', EASY_TRANSLATE),
            __('String translations', EASY_TRANSLATE),
            self::CAPABILITY,
            self::MENU_PARTIAL_TRANSLATIONS,
            [$this, 'show_partial_translations_page']
        );

        add_submenu_page(
            self::MENU_MAIN,
            __('Projects', EASY_TRANSLATE),
            __('Projects', EASY_TRANSLATE),
            self::CAPABILITY,
            self::MENU_PROJECTS,
            [$this, 'show_projects_page']
        );

        add_submenu_page(
            self::MENU_MAIN,
            __('Integration', EASY_TRANSLATE),
            __('Integration', EASY_TRANSLATE),
            self::CAPABILITY,
            self::MENU_INTEGRATION,
            [$this->api_integration, 'include_api_integration_partial']
        );
    }

    /**
     * Register the form fields for the API integration page.
     */
    public function init_integration()
    {
        $this->api_integration->init();
    }

    /**
     * Screen options for the string translation list table.
     */
    public function load_string_translation_list_screen_options()
    {

        $this->string_translation_list = new Easy_Translate_String_Translation_List();
        $this->string_translation_list->load_screen_options();

    }

    /**
     * Filters a screen option value before it is set.
     *
     * @param $status
     * @param $option
     * @param $value
     * @return int
     */
    public function set_string_translation_list_screen_option($status, $option, $value)
    {

        return ($option == Easy_Translate_String_Translation_List::PER_PAGE_OPTION) ? (int)$value : $status;

    }

    /**
     * Screen options for the post translation list table.
     */
    public function load_post_translation_list_screen_options()
    {

        $this->post_translation_list = new Easy_Translate_Post_Translation_List();
        $this->post_translation_list->load_screen_options();

    }

    /**
     * Filters a screen option value before it is set.
     *
     * @param $status
     * @param $option
     * @param $value
     * @return int
     */
    public function set_post_translation_list_screen_option($status, $option, $value)
    {

        return ($option == Easy_Translate_Post_Translation_List::PER_PAGE_OPTION) ? (int)$value : $status;

    }

    /**
     * The completed task action callback.
     */
    public function completed_task_callback()
    {
        $translation = new Easy_Translate_Translation();
        $translation->completed_task_callback();
    }

    /**
     * Render string translations page.
     */
    public function show_partial_translations_page()
    {

        $this->string_translation_list->prepare_items();
        $this->string_translation_list->render();

    }

    /**
     * Render translations page.
     */
    public function show_translations_page()
    {
        $this->post_translation_list->prepare_items();
        $this->post_translation_list->render();
    }

    /**
     * Render projects page.
     */
    public function show_projects_page()
    {

        $this->projects->render();

    }

    /**
     * Update existing translation.
     *
     * @param $post_ID
     */
    public function edit_post_hook($post_ID)
    {

        $updater = new Easy_Translate_Translation_Updater();
        $updater->run($post_ID);

    }

    /**
     * Includes plugin functions.
     */
    public function include_plugin_functions()
    {

        include_once EASY_TRANSLATE_ROOT . 'admin/functions.php';

    }

    /**
     * Set post auto translate value.
     */
    public function set_auto_translate_callback()
    {

        Easy_Translate_Translation_List::set_auto_translate();

    }

}
