<?php

/**
 * Class Easy_Translate_Post_Translation_List.
 *
 * @package    Easy_Translate
 * @subpackage Easy_Translate/admin
 */
class Easy_Translate_Post_Translation_List extends Easy_Translate_Translation_List
{

    /**
     * Per page item option.
     */
    const PER_PAGE_OPTION = 'edit_post_translation_per_page';

    /**
     * Excluded post types.
     */
    const EXCLUDED_POST_TYPES = [
        'attachment',
        'revision',
        'nav_menu_item',
        'custom_css',
        'customize_changeset',
        'oembed_cache',
        'user_request',
    ];

    /**
     * Easy_Translate_Post_Translation_List constructor.
     */
    public function __construct()
    {

        add_filter('manage_' . get_current_screen()->id . '_columns', [$this, 'manage_columns']);

        parent::__construct([
            'singular' => __('translation', EASY_TRANSLATE),
            'plural'   => __('translations', EASY_TRANSLATE),
            'ajax'     => false,
        ]);

    }

    /**
     * Render screen options column.
     *
     * @param $columns
     * @return mixed
     */
    public function manage_columns($columns)
    {

        $columns['status'] = __('Status', EASY_TRANSLATE);

        return $columns;

    }

    /**
     * Get a list of sortable columns.
     *
     * @return array
     */
    protected function get_sortable_columns()
    {

        $sortable_columns = [
            'post_title' => 'post_title',
            'post_type'  => 'post_type',
        ];

        return $sortable_columns;

    }

    /**
     * Get a list of all, hidden and sortable columns, with filter applied
     *
     * @return array
     */
    public function get_column_info()
    {

        // $_column_headers is already set / cached.
        if (isset($this->_column_headers) && is_array($this->_column_headers)) {
            // Back-compat for list tables that have been manually setting $_column_headers for horse reasons.
            // In 4.3, we added a fourth argument for primary column.
            $column_headers = [[], [], [], $this->get_primary_column_name()];
            foreach ($this->_column_headers as $key => $value) {
                $column_headers[$key] = $value;
            }

            return $column_headers;
        }

        $columns = $this->get_columns();
        $hidden = get_hidden_columns($this->screen);

        $sortable_columns = $this->get_sortable_columns();
        /**
         * Filters the list table sortable columns for a specific screen.
         * The dynamic portion of the hook name, `$this->screen->id`, refers
         * to the ID of the current screen, usually a string.
         *
         * @param array $sortable_columns An array of sortable columns.
         * @since 3.5.0
         */
        $_sortable = apply_filters("manage_{$this->screen->id}_sortable_columns", $sortable_columns);

        $sortable = [];
        foreach ($_sortable as $id => $data) {
            if (empty($data)) {
                continue;
            }

            $data = (array)$data;
            if (!isset($data[1])) {
                $data[1] = false;
            }

            $sortable[$id] = $data;
        }

        $primary = $this->get_primary_column_name();
        $this->_column_headers = [$columns, $hidden, $sortable, $primary];

        return $this->_column_headers;

    }

    /**
     * Get a list of columns.
     *
     * @return array
     */
    public function get_columns()
    {

        $table_columns = [
            'cb'             => '<input type="checkbox" />', // to display the checkbox.
            'post_title'     => __('Title', EASY_TRANSLATE),
            'post_type'      => __('Type', EASY_TRANSLATE),
            'status'         => self::get_translation_language_flag($this->translation_languages),
            'auto_translate' => __('Auto translate ON/OFF', EASY_TRANSLATE),
        ];

        return $table_columns;

    }

    /**
     * Fetch table data from the WordPress database.
     *
     * @return array|object|null
     */
    public function fetch_table_data()
    {

        global $wpdb;
        $a_where = [
            'po.post_status <> "trash"',
            'po.post_status <> "auto-draft"',
            'tr.element_type LIKE "post_%"',
        ];

        foreach (self::EXCLUDED_POST_TYPES as $post_type) {
            $a_where[] = "po.post_type <> '{$post_type}'";
        }

        if (isset($_GET['post_type']) && $_GET['post_type'] != '') {
            $a_where[] = 'post_type = "' . esc_sql($_GET['post_type']) . '"';
        }

        $lang = $this->get_current_admin_language();
        if ($lang !== 'all') {
            $a_where[] = 'tr.language_code = "' . $lang . '"';
        }

        $orderby = (isset($_GET['orderby'])) ? esc_sql($_GET['orderby']) : 'post_modified';
        $order = (isset($_GET['order'])) ? esc_sql($_GET['order']) : 'DESC';

        $join = " LEFT JOIN {$wpdb->prefix}icl_translations AS tr ON tr.element_id = po.ID";

        $query = "SELECT po.ID, po.post_title, po.post_status, po.post_type, tr.language_code FROM {$wpdb->prefix}posts AS po {$join} WHERE " . implode(" AND ",
                $a_where) . " ORDER BY po.{$orderby} {$order};";

        return $wpdb->get_results($query, ARRAY_A);

    }

    /**
     * Render a column when no column specific method exists.
     *
     * @param array  $item
     * @param string $column_name
     * @return mixed
     */
    public function column_default($item, $column_name)
    {

        switch ($column_name) {
            case 'post_title':
                return self::get_translation_language_flag([$item['language_code']]) . '&nbsp;' . $item[$column_name];
            default:
                return $item[$column_name];
        }

    }

    /**
     * Get value for checkbox column.
     *
     * @param object $item A row's data
     * @return string Text to be placed inside the column <td>.
     */
    protected function column_cb($item)
    {

        return sprintf(
            "<input type='checkbox' name='posts[]' id='post_{$item['ID']}' value='{$item['ID']}' />"
        );

    }

    /**
     * Get value for auto translate column.
     *
     * @param object $item A row's data
     * @return string Text to be placed inside the column <td>.
     */
    protected function column_auto_translate($item)
    {
        $checked_value = get_post_meta($item['ID'], 'et_auto_translate', true) ? 'checked' : '';

        return sprintf(
            '<label for="%s" class="switch"><input id="%s" value="%s" class="js-auto-translate" type="checkbox" %s><span class="slider round"></span></label>',
            'auto_translate_' . $item['ID'],
            'auto_translate_' . $item['ID'],
            $item['ID'],
            $checked_value
        );

    }

    /**
     * Get value for status column.
     *
     * @param $item
     * @return string
     */
    protected function column_status($item)
    {

        $statuses = '';
        $post_id = $item['ID'];
        $translation_status = $this->get_translation_status($post_id);
        foreach ($this->translation_languages as $lang) {
            if (isset($translation_status[$lang])) {
                switch ($translation_status[$lang]) {
                    case self::STATUS_WAITING:
                        $statuses .= '<span><i class="otgs-ico-waiting" title="' . $lang . ': ' . __('Waiting for translator',
                                EASY_TRANSLATE) . '"></i></span>';
                        break;
                    case self::STATUS_NEEDS_UPDATE:
                        $statuses .= '<span><i class="otgs-ico-needs-update" title="' . $lang . ': ' . __('Translation needs update',
                                EASY_TRANSLATE) . '"></i></span>';
                        break;
                    case self::STATUS_DUPLICATE:
                        $statuses .= '<span><i class="otgs-ico-duplicate" title="' . $lang . ': ' . __('Duplicate',
                                EASY_TRANSLATE) . '"></i></span>';
                        break;
                    case self::STATUS_COMPLETE:
                        $statuses .= '<span><i class="otgs-ico-translated" title="' . $lang . ': ' . __('Complete',
                                EASY_TRANSLATE) . '"></i></span>';
                        break;
                    default:
                        $statuses .= '<span><i class="otgs-ico-not-translated" title="' . $lang . ': ' . __('Not translated',
                                EASY_TRANSLATE) . '"></i></span>';
                }
            } else {
                $statuses .= '<span><i class="otgs-ico-not-translated" title="' . $lang . ': ' . __('Not translated',
                        EASY_TRANSLATE) . '"></i></span>';
            }
        }

        return $statuses;

    }

    /**
     * Process actions triggered by the user
     */
    public function handle_table_actions()
    {

        /*
         * Note: Table bulk_actions can be identified by checking $_REQUEST['action'] and $_REQUEST['action2']
         *
         * action - is set if checkbox from top-most select-all is set, otherwise returns -1
         * action2 - is set if checkbox the bottom-most select-all checkbox is set, otherwise returns -1
         */

        // check for individual row actions
        $the_table_action = $this->current_action();

        // check for table bulk actions
        if (isset($_REQUEST['action']) && isset($_REQUEST['posts']) && isset($_REQUEST['target_languages'])) {

            $nonce = wp_unslash($_REQUEST['_wpnonce']);
            // verify the nonce.
            /*
             * Note: the nonce field is set by the parent class
             * wp_nonce_field( 'bulk-' . $this->_args['plural'] );
             *
             */
            if (!wp_verify_nonce($nonce, 'bulk-translations')) {
                $this->invalid_nonce_redirect();
            } else {
                $a_string_id = [];
                $content = [];

                foreach ($_REQUEST['posts'] as $post_id) {
                    $content[] = $this->get_post_content($post_id, $a_string_id);
                }

                $urlParts = parse_url(home_url());
                $hostName = $urlParts['host'] ?? get_site_url();
                $name = '[' . date('Y-m-d') . '] ' . $hostName . ' page translations';

                $workflow = 'translation';
                if ($_REQUEST['workflow'] === 'self+machine_translation') {
                    $workflow = 'self+machine_translation';
                }

                if ($content[0]) {
                    $options = get_option(EASY_TRANSLATE_INTEGRATION_OPTION);
                    $service = new Easy_Translate_API_Laas(Easy_Translate_API_Integration::map_fields($options));
                    $response = $service->translate($this->source_language, array_keys($_REQUEST['target_languages']),
                        $content, $workflow, $name);
                    if ($response['error'] ?? false) {
                        add_settings_error(
                            '',
                            'translate-error',
                            $response['error'] ?? '',
                            'error'
                        );

                        return null;
                    }
                    add_settings_error(
                        '',
                        'translate-success',
                        __('Sent successfully!', EASY_TRANSLATE),
                        'success'
                    );
                    $this->update_string_status($a_string_id, self::STATUS_WAITING);
                    $this->update_duplicate_status($_REQUEST['posts'], array_keys($_REQUEST['target_languages']),
                        self::STATUS_WAITING);
                } else {
                    add_settings_error(
                        '',
                        'translate-error',
                        __('Unable to get content for translation', EASY_TRANSLATE),
                        'error'
                    );
                }
            }
        }
    }

    /**
     * Register and configure list table screen option.
     */
    public function load_screen_options()
    {

        add_screen_option('per_page', [
            'label'   => __('Items per page', EASY_TRANSLATE),
            'default' => 20,
            'option'  => self::PER_PAGE_OPTION,
        ]);

    }

    /**
     * Extra controls to be displayed between bulk actions and pagination.
     *
     * @param string $which
     */
    protected function extra_tablenav($which)
    {

        if ($which == 'top') {
            $current_post_type = $_GET['post_type'] ?? '';
            $post_types = $this->get_post_types();
            ?>

          <label for="post-type"><?php echo __('Select post type: ', EASY_TRANSLATE) ?></label>
          <select name="post_type" id="post-type">
            <option <?php selected($current_post_type, ''); ?>
              value=""><?php echo __('All post types', EASY_TRANSLATE) ?></option>
              <?php
              foreach ($post_types as $post_type) {
                  ?>
                <option <?php selected($current_post_type, $post_type); ?> value="<?= $post_type ?>">
                    <?= $post_type ?>
                </option>
                  <?php
              }
              ?>

          </select>

            <?php
            submit_button(__('Filter', EASY_TRANSLATE), '', '', false);
        }

    }

    /**
     * Returns post types.
     *
     * @return string[]
     */
    protected function get_post_types()
    {

        $post_types = get_post_types();
        foreach (self::EXCLUDED_POST_TYPES as $post_type) {
            unset($post_types[$post_type]);
        }

        return $post_types;

    }

    /**
     * Render the list table.
     */
    protected function render_list()
    {

        include_once(EASY_TRANSLATE_ROOT . 'admin/partials/post-translation.php');

    }

    /**
     * Returns per page option.
     *
     * @return string
     */
    protected function get_per_page_option()
    {

        return self::PER_PAGE_OPTION;

    }

}
